#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdint.h>
#include "common.h"
#include "base64.h"

void __CheckFunc__(bool iFlag, const char* const file, int line, const uint8_t* const msg )
{
    if(!iFlag)
    {
        if(msg)
        {
            fprintf(stderr, "ERROR[%s:%d]:%s\n", file, line, msg);
            exit(1);
        }
        else
        {
            fprintf(stderr, "ERROR[%s:%d]\n", file, line);
            exit(1);
        }
    }
}

unsigned long read_file_to_buffer(	const char* const filepath,
									uint8_t** obuff,
									bool remove_new_line_chr)
{
	if(NULL==obuff || NULL==filepath)
		return 0;

	FILE* fh=0;
	size_t len=0;
	char* line=NULL;
	unsigned long total_len=0;

	if( (fh=fopen(filepath, "rb")) == 0 )
	{
		fprintf(stderr, "File %s doesn't exist. Exiting...\n", filepath);
        goto end;
	}

	if(NULL==(*obuff))
	{
		//allocate
		fseek(fh, 0, SEEK_END);
		int pos = ftell(fh);
		fseek(fh, 0, SEEK_SET);
		*obuff = (uint8_t*) malloc(pos);
	}

	while( getline(&line, &len, fh) != -1 )
	{
		len = strlen(line); // OZAPTF: needed? you have len above
		if(remove_new_line_chr)
			len--;

		memcpy((*obuff)+total_len, line, len);
		total_len+=len;
    	free(line);
		line=NULL;
		len=0;
	}
    free(line);

end:
    fclose(fh);
	return total_len;
}

Result_t load_base64_to_hex(const char* const filepath,
							uint8_t** hex_buffer,
							size_t* len)
{
	if(hex_buffer == NULL)
		return Result_Error;

	uint8_t* tmpbuff = NULL;
	*len = read_file_to_buffer(filepath, &tmpbuff, 1);
	if(*len == 0)
		return Result_Error;
	*hex_buffer = (uint8_t*) malloc(*len);
	base64_to_hex(tmpbuff, *len, *hex_buffer);
	free(tmpbuff);

	return Result_OK;
}


// checks if there are identical blocks in the i_textline
int same_blocks(const char* i_textline, const int i_bs, int* first_block, int* second_block)
{
    int chunk_nb = strlen(i_textline)/i_bs;
    for(int i=0; i<chunk_nb-1; i++)
    {
        for(int j=i+1; j<chunk_nb; j++)
        {
            if(memcmp(&i_textline[i*i_bs], &i_textline[j*i_bs],i_bs)==0)
            {
                *first_block = i+1;
                *second_block = j+1;
                return 1;
            }
        }
    }
    return 0;
}

void CryptoAttribs_t::Init(CryptoAttribs_t* ctx)
{
	ctx->iv = NULL;
	ctx->iv_len = 0;
	ctx->output = NULL;
	ctx->output_len = 0;
	ctx->input = NULL;
	ctx->input_len = 0;
	ctx->operation = kEncrypt;
    ctx->padding = kPadding_None;
}

void CryptoAttribs_t::Free(CryptoAttribs_t* ctx)
{
	if(ctx->iv!=NULL)
	{
		::free(ctx->iv);
		ctx->iv=NULL;
	}
	if(ctx->output!=NULL)
	{
		::free(ctx->output);
		ctx->output=NULL;
	}
	if(ctx->input!=NULL)
	{
		::free(ctx->input);
		ctx->input=NULL;
	}
	ctx->iv_len = 0;
	ctx->output_len = 0;
	ctx->input_len = 0;
}

void Key_t::Init(Key_t* ctx)
{
	ctx->key = NULL;
	ctx->len = 0;
}


void Key_t::Free(Key_t* ctx)
{
	if(ctx->key!=NULL)
    {
		::free(ctx->key);
        ctx->key = NULL;
    }
	ctx->len = 0;
}
