#ifndef __common_h__
#define __common_h__

#include <stddef.h>
#include <stdint.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>

// Functions for logging start/stop of the test cases
#define TCASE(x) void x() { printf("Entering %s\n", __func__);
#define TCASE_E }

// When iFlag is false then function logs an error and halts program
void __CheckFunc__(bool iFlag, const char* const file, int line, const uint8_t* const msg = NULL );
#define __CHECK2__(Arg1,Msg) __CheckFunc__(Arg1,__FILE__,__LINE__, Msg)
#define __CHECK1__(Arg1) __CheckFunc__(Arg1,__FILE__,__LINE__)
#define __CHECK_MACRO_CHOOSER__(_1,_2, MACRO,...) MACRO
#define CHECK(...) __CHECK_MACRO_CHOOSER__(__VA_ARGS__, __CHECK2__, __CHECK1__)(__VA_ARGS__)

// Some useful defines
#define MIN(a,b) a<b ? a : b
#define ROUNDUP_16_BYTES(X) (((X+15)/16) * 16)
#define AES_BLOCK_SIZE 16

#define OP_CHECK(exp)                                           \
    do {                                                        \
        if(!exp) {                                              \
            printf("%s:%d Error with expression %s %d\n", __FILE__, __LINE__, #exp, exp); \
            goto end;                                           \
        }                                                       \
    }                                                           \
    while(false)

typedef enum
{
    Result_OK = 0,
    Result_Error
} Result_t;

typedef enum {
    kEncrypt = 0,
    kDecrypt = 1
} CryptoOperation;

typedef enum {
  kPadding_None = 0,
  kPadding_PKCS7
} PaddingType;

typedef struct CtrCtx_t {
  uint8_t nonce[8];
  uint8_t counter[8];
  uint8_t encrypted_counter[AES_BLOCK_SIZE];
} CtrCtx_t;

typedef struct ModeCtx_t {
  uint8_t iv[AES_BLOCK_SIZE];
  CtrCtx_t ctr;
} ModeCtx_t;

/* -------------------------------------------------------------------------
   Structure for key passing
   -------------------------------------------------------------------------*/
typedef struct Key_t {
  uint8_t* key;
  size_t   len;

  static void Init(Key_t* ctx);
  static void Free(Key_t* ctx);
} Key_t;

/* -------------------------------------------------------------------------
   Struct used for symmetric encryption
   -------------------------------------------------------------------------*/
struct  CryptoAttribs_t {
	uint8_t* input;
	size_t	 input_len;
	uint8_t* output;
	size_t	 output_len;
	uint8_t* iv;
	size_t   iv_len;
  uint64_t output_cursor;

	CryptoOperation  operation;
  ModeCtx_t mode;
  // If requested, padding will align produced block size with key size
  PaddingType padding;
  Key_t key;

	static void Init(CryptoAttribs_t* ctx);
	static void Free(CryptoAttribs_t* ctx);
};

struct int24 {
	unsigned int data:24;
};


/* -------------------------------------------------------------------------
   Misclleneus utilities
   -------------------------------------------------------------------------*/
unsigned long read_file_to_buffer(	const char* const filepath,
									uint8_t** obuff,
									bool remove_new_line_chr = false);

Result_t load_base64_to_hex(const char* const filepath, uint8_t** hex_buffer, size_t* len);

int same_blocks(const char* i_textline, const int i_bs, int* first_block, int* second_block);

#endif // __common_h__
